# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

from __future__ import unicode_literals

import logging

from django.utils.translation import gettext_lazy as _


class BlueException(Exception):

    ERROR_CODE = "0000000"
    MESSAGE = _("APP异常")
    STATUS_CODE = 500
    LOG_LEVEL = logging.ERROR

    def __init__(self, message=None, data=None, *args):
        """
        :param message: 错误消息
        :param data: 其他数据
        :param context: 错误消息 format dict
        :param args: 其他参数
        """
        super(BlueException, self).__init__(*args)
        self.message = self.MESSAGE if message is None else message
        self.data = data

    def render_data(self):
        return self.data

    def response_data(self):
        return {
            "result": False,
            "code": self.ERROR_CODE,
            "message": self.message,
            "data": self.render_data(),
        }


class ClientBlueException(BlueException):

    MESSAGE = _("客户端请求异常")
    ERROR_CODE = "40000"
    STATUS_CODE = 400


class ServerBlueException(BlueException):

    MESSAGE = _("服务端服务异常")
    ERROR_CODE = "50000"
    STATUS_CODE = 500


class ResourceNotFound(ClientBlueException):

    MESSAGE = _("找不到请求的资源")
    ERROR_CODE = "40400"
    STATUS_CODE = 404


class ParamValidationError(ClientBlueException):

    MESSAGE = _("参数验证失败")
    ERROR_CODE = "40000"
    STATUS_CODE = 400


class ParamRequired(ClientBlueException):

    MESSAGE = _("关键参数缺失")
    ERROR_CODE = "40001"
    STATUS_CODE = 400


class AccessForbidden(ClientBlueException):

    MESSAGE = _("登陆失败")
    ERROR_CODE = "40301"
    STATUS_CODE = 403


class RequestForbidden(ClientBlueException):

    MESSAGE = _("请求拒绝")
    ERROR_CODE = "40320"
    STATUS_CODE = 403


class ResourceLock(ClientBlueException):

    MESSAGE = _("请求资源被锁定")
    ERROR_CODE = "40330"
    STATUS_CODE = 403


class MethodError(ClientBlueException):

    MESSAGE = _("请求方法不支持")
    ERROR_CODE = "40501"
    STATUS_CODE = 405


class RioVerifyError(ClientBlueException):

    MESSAGE = _("登陆请求经智能网关检测失败")
    ERROR_CODE = "40101"
    STATUS_CODE = 401


class BkJwtVerifyError(ClientBlueException):

    MESSAGE = _("登陆请求经JWT检测失败")
    ERROR_CODE = "40102"
    STATUS_CODE = 401


class DatabaseError(ServerBlueException):

    MESSAGE = _("数据库异常")
    ERROR_CODE = "50110"


class ApiNetworkError(ServerBlueException):

    MESSAGE = _("网络异常导致远程服务失效")
    ERROR_CODE = "50301"
    STATUS_CODE = 503


class ApiResultError(ServerBlueException):

    MESSAGE = _("远程服务请求结果异常")
    ERROR_CODE = "50302"
    STATUS_CODE = 503


class ApiNotAcceptable(ServerBlueException):

    MESSAGE = _("远程服务返回结果格式异常")
    ERROR_CODE = "50303"
    STATUS_CODE = 503



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import logging

from django.conf import settings
from django.contrib import auth
from django.utils.deprecation import MiddlewareMixin

from blueapps.account.conf import ConfFixture
from blueapps.account.handlers.response import ResponseHandler

logger = logging.getLogger("component")


class BkJwtLoginRequiredMiddleware(MiddlewareMixin):
    def process_view(self, request, view, args, kwargs):
        """
        可通过登录认证的请求：
        1. 带有BK JWT HEADER
        2. JWT签名正确
        """
        # 框架前置中间件，已将识别的客户端信息填充进 request
        if not hasattr(request, "is_bk_jwt") or not request.is_bk_jwt():
            return None

        logger.debug("当前请求是否经过JWT转发")
        login_exempt = getattr(view, "login_exempt", False)

        # 每次请求都需要做校验
        if not (login_exempt or request.user.is_authenticated):
            user = auth.authenticate(request=request)
            if user and user.username != request.user.username:
                auth.login(request, user)
            if request.user.is_authenticated:
                # 登录成功，确认登陆正常后退出
                return None
            handler = ResponseHandler(ConfFixture, settings)
            return handler.build_bk_jwt_401_response(request)
        return None

    def process_response(self, request, response):
        return response



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import os

from blueapps.conf.default_settings import APP_CODE, BASE_DIR


def get_paas_v2_logging_config_dict(is_local, bk_log_dir, log_level):
    """
    日志V2对外版设置
    """

    app_code = os.environ.get("APP_ID", APP_CODE)

    # 设置日志文件夹路径
    if is_local:
        log_dir = os.path.join(os.path.dirname(BASE_DIR), "logs", app_code)
    else:
        log_dir = os.path.join(os.path.join(bk_log_dir, app_code))

    # 如果日志文件夹不存在则创建,日志文件存在则延用
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    return {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "simple": {"format": "%(levelname)s %(message)s \n"},
            "verbose": {
                "format": "%(levelname)s [%(asctime)s] %(pathname)s "
                "%(lineno)d %(funcName)s %(process)d %(thread)d "
                "\n \t %(message)s \n",
                "datefmt": "%Y-%m-%d %H:%M:%S",
            },
        },
        "handlers": {
            "component": {
                "class": "logging.handlers.RotatingFileHandler",
                "formatter": "verbose",
                "filename": os.path.join(log_dir, "component.log"),
                "maxBytes": 1024 * 1024 * 10,
                "backupCount": 5,
            },
            "console": {
                "level": "DEBUG",
                "class": "logging.StreamHandler",
                "formatter": "simple",
            },
            "null": {"level": "DEBUG", "class": "logging.NullHandler"},
            "root": {
                "class": "logging.handlers.RotatingFileHandler",
                "formatter": "verbose",
                "filename": os.path.join(log_dir, "%s.log" % app_code),
                "maxBytes": 1024 * 1024 * 10,
                "backupCount": 5,
            },
            "wb_mysql": {
                "class": "logging.handlers.RotatingFileHandler",
                "formatter": "verbose",
                "filename": os.path.join(log_dir, "wb_mysql.log"),
                "maxBytes": 1024 * 1024 * 4,
                "backupCount": 5,
            },
        },
        "loggers": {
            # V2旧版开发框架使用的logger
            "component": {
                "handlers": ["component"],
                "level": "WARNING",
                "propagate": True,
            },
            "django": {"handlers": ["null"], "level": "INFO", "propagate": True},
            "django.server": {
                "handlers": ["console"],
                "level": log_level,
                "propagate": True,
            },
            "django.request": {
                "handlers": ["console"],
                "level": "ERROR",
                "propagate": True,
            },
            "django.db.backends": {
                "handlers": ["wb_mysql"],
                "level": log_level,
                "propagate": True,
            },
            "root": {"handlers": ["root"], "level": log_level, "propagate": True},
            # V3新版使用的日志
            "celery": {"handlers": ["root"], "level": log_level, "propagate": True},
            "blueapps": {"handlers": ["root"], "level": log_level, "propagate": True},
            "app": {"handlers": ["root"], "level": log_level, "propagate": True},
        },
    }



# -*- coding: utf-8 -*-
"""
Tencent is pleased to support the open source community by making 蓝鲸智云PaaS平台社区版 (BlueKing PaaS Community
Edition) available.
Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
specific language governing permissions and limitations under the License.
"""

import logging

from bkoauth import get_access_token_by_user

from blueapps.account import get_user_model

logger = logging.getLogger("blueapps")


def get_component_client_common_args():
    """
    获取ComponentClient需要的common_args
    @return:
    {
        access_token = 'xxx'
    }
    @rtype: dict
    """
    try:
        last_login_user = get_user_model().objects.all().order_by("-last_login")[0]
    except IndexError:
        logger.warn("There is not a last_login_user")
        raise IndexError("There is not a last_login_user")
    access_token = get_access_token_by_user(last_login_user.username).access_token
    return dict(access_token=access_token)
